/*****************************************************************************
 * HASM64                                                                    *
 *****************************************************************************/

/*****************************************************************************
 * File name:     scan64.h                                                   *
 * Created:       2001-05-26 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2004-05-31 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2001-2004 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by  the Free Software Foundation.                                         *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: scan64.h 20 2004-05-31 14:54:03Z hampa $ */


#ifndef HASM64_SCAN64_H
#define HASM64_SCAN64_H 1


#include "hasm64.h"


#define TOK_IDENT  16
#define TOK_DIGIT  17
#define TOK_STRING 18

#define TOK_SYM_MODULE  100
#define TOK_SYM_END     101
#define TOK_SYM_ORG     102
#define TOK_SYM_INCLUDE 103

#define TOK_SYM_REL_START 200
#define TOK_SYM_EQU       200
#define TOK_SYM_NEQU      201
#define TOK_SYM_LESS      202
#define TOK_SYM_GRTR      203
#define TOK_SYM_LEQU      204
#define TOK_SYM_GEQU      205
#define TOK_SYM_REL_END   205

#define TOK_SYM_BIT_START 300
#define TOK_SYM_OR        300
#define TOK_SYM_AND       301
#define TOK_SYM_XOR       302
#define TOK_SYM_BIT_END   302

#define TOK_SYM_SUM_START 400
#define TOK_SYM_PLUS      400
#define TOK_SYM_MINUS     401
#define TOK_SYM_SUM_END   401

#define TOK_SYM_SHIFT_START 500
#define TOK_SYM_SHL         500
#define TOK_SYM_SHR         501
#define TOK_SYM_SHIFT_END   501

#define TOK_SYM_MUL_START 600
#define TOK_SYM_MUL       600
#define TOK_SYM_DIV       601
#define TOK_SYM_MOD       602
#define TOK_SYM_MUL_END   602

#define TOK_SYM_LPAR      800
#define TOK_SYM_RPAR      801
#define TOK_SYM_COLON     802
#define TOK_SYM_PERIOD    803
#define TOK_SYM_COMMA     804
#define TOK_SYM_LSPAR     805
#define TOK_SYM_RSPAR     806
#define TOK_SYM_NOT       807
#define TOK_SYM_LO        808
#define TOK_SYM_HI        809
#define TOK_SYM_IMM       810

#define TOK_SYM_A         900
#define TOK_SYM_X         901
#define TOK_SYM_Y         902
#define TOK_SYM_PC        903

#define TOK_MNEMONIC_START 1000
#define TOK_OP_ADC         1000
#define TOK_OP_AND         1001
#define TOK_OP_ASL         1002
#define TOK_OP_BCC         1003
#define TOK_OP_BCS         1004
#define TOK_OP_BEQ         1005
#define TOK_OP_BNE         1006
#define TOK_OP_BPL         1007
#define TOK_OP_BMI         1008
#define TOK_OP_BVC         1009
#define TOK_OP_BVS         1010
#define TOK_OP_BIT         1011
#define TOK_OP_BRK         1012
#define TOK_OP_CLC         1013
#define TOK_OP_CLD         1014
#define TOK_OP_CLI         1015
#define TOK_OP_CLV         1016
#define TOK_OP_CMP         1017
#define TOK_OP_CPX         1018
#define TOK_OP_CPY         1019
#define TOK_OP_DB          1020
#define TOK_OP_DEC         1021
#define TOK_OP_DEX         1022
#define TOK_OP_DEY         1023
#define TOK_OP_EOR         1024
#define TOK_OP_INC         1025
#define TOK_OP_INX         1026
#define TOK_OP_INY         1027
#define TOK_OP_JMP         1028
#define TOK_OP_JSR         1029
#define TOK_OP_LDA         1030
#define TOK_OP_LDX         1031
#define TOK_OP_LDY         1032
#define TOK_OP_LSR         1033
#define TOK_OP_NOP         1034
#define TOK_OP_ORA         1035
#define TOK_OP_PHA         1036
#define TOK_OP_PHP         1037
#define TOK_OP_PLA         1038
#define TOK_OP_PLP         1039
#define TOK_OP_RB          1040
#define TOK_OP_ROL         1041
#define TOK_OP_ROR         1042
#define TOK_OP_RTI         1043
#define TOK_OP_RTS         1044
#define TOK_OP_SBC         1045
#define TOK_OP_SEC         1046
#define TOK_OP_SED         1047
#define TOK_OP_SEI         1048
#define TOK_OP_STA         1049
#define TOK_OP_STX         1050
#define TOK_OP_STY         1051
#define TOK_OP_TAX         1052
#define TOK_OP_TAY         1053
#define TOK_OP_TSX         1054
#define TOK_OP_TXA         1055
#define TOK_OP_TXS         1056
#define TOK_OP_TYA         1057
#define TOK_OP_REP         1058
#define TOK_MNEMONIC_END   1059


typedef struct {
  unsigned           tok;
  char               *str;
} keyword_t;


typedef struct {
  scanner_t     *next;

  char          *fname;
  FILE          *fp;

  unsigned char buf[1024];
  unsigned      buf_i;
  unsigned      buf_n;

  unsigned      keycnt;
  keyword_t     *keywrd;
} scan64_t;


/*!************************************************************************
 * Create a new hasm64 scanner.
 **************************************************************************/
scanner_t *scn64_new (void);

/*!************************************************************************
 * Delete a hasm64 scanner.
 **************************************************************************/
void scn64_del (scanner_t *scn);

/*!************************************************************************
 * Set the file to scan. This will reset the scanner. Returns 0 if
 * the file could be opened.
 **************************************************************************/
int scn64_set_file (scanner_t *scn, char *fname);

/*!************************************************************************
 * Get the file name of the file being scanned.
 **************************************************************************/
char *scn64_get_file (scanner_t *scn);

/*!************************************************************************
 * Set the next scanner in list.
 **************************************************************************/
void scn64_set_next (scanner_t *scn, scanner_t *next);

/*!************************************************************************
 * Get the next scanner in list.
 **************************************************************************/
scanner_t *scn64_get_next (scanner_t *scn);

/*!************************************************************************
 * Scan the next token.
 **************************************************************************/
void scn64_scan (scanner_t *scn);


#endif
