/*****************************************************************************
 * HASM64                                                                    *
 *****************************************************************************/

/*****************************************************************************
 * File name:     symtab.h                                                   *
 * Created:       1997-09-04 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2004-05-31 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 1997-2004 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: symtab.h 21 2004-05-31 15:34:10Z hampa $ */


#ifndef HASM64_SYMTAB_H
#define HASM64_SYMTAB_H 1


/*!***************************************************************************
 * @short The symbol type
 *****************************************************************************/
typedef struct symbol_s {
  struct symbol_s *next;
  char            *name;
  unsigned        val;
  unsigned        pass;
} symbol_t;


/*!***************************************************************************
 * @short The symbol table type
 *****************************************************************************/
typedef struct {
  unsigned cnt;
  symbol_t *sym;
} symtab_t;


/*!***************************************************************************
 * @short Create and initialize a new symbol with name and value.
 *****************************************************************************/
symbol_t *sym_new (const char *name, unsigned val);

/*!***************************************************************************
 * @short Delete a symbol.
 *****************************************************************************/
void sym_del (symbol_t *sym);

/*!***************************************************************************
 * @short Set the symbol value.
 *****************************************************************************/
void sym_set_val (symbol_t *sym, unsigned val);

/*!***************************************************************************
 * @short Get the symbol value.
 *****************************************************************************/
unsigned sym_get_val (symbol_t *sym);

/*!***************************************************************************
 * @short Set the symbol name.
 *****************************************************************************/
void sym_set_name (symbol_t *sym, const char *name);

/*!***************************************************************************
 * @short Get the symbol name.
 *****************************************************************************/
const char *sym_get_name (symbol_t *sym);


/*!***************************************************************************
 * @short Initialize a symbol table
 *****************************************************************************/
void st_init (symtab_t *st);

/*!***************************************************************************
 * @short Create and initialize a new empty symbol table.
 *****************************************************************************/
symtab_t *st_new (void);

/*!***************************************************************************
 * @short Free the resources used by a symbol table
 *****************************************************************************/
void st_free (symtab_t *st);

/*!***************************************************************************
 * @short Delete a symbol table and all the symbols in it.
 *****************************************************************************/
void st_del (symtab_t *st);

/*!***************************************************************************
 * @short Add a symbol to the symbol table.
 *****************************************************************************/
void st_sym_add (symtab_t *st, symbol_t *sym);

/*!***************************************************************************
 * @short Remove a symbol from the symbol table.
 *****************************************************************************/
void st_sym_rm (symtab_t *st, symbol_t *sym);

/*!***************************************************************************
 * @short Find a symbol in the symbol table.
 * @return A pointer to the symbol or NULL on error
 *****************************************************************************/
symbol_t *st_sym_get (symtab_t *st, const char *name);

/*!***************************************************************************
 * @short Dump the symbol table to a file
 *****************************************************************************/
int st_dump (symtab_t *st, FILE *fp);

/*!***************************************************************************
 * @short Dump the symbol table to a file
 *****************************************************************************/
int st_dump_file (symtab_t *st, char *fname);


#endif
